package de.hmmh.pmt.util;

import de.hmmh.pmt.db.Allocation;
import de.hmmh.pmt.db.Project;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validation;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Set;

@Component
public class Validator {

    public boolean isValidProject(Project project) {
        jakarta.validation.Validator validator = Validation
                .buildDefaultValidatorFactory()
                .getValidator();
        Set<ConstraintViolation<Project>> violations = validator.validate(project);

        LocalDateTime start = project.getStart();
        LocalDateTime plannedEnd = project.getPlannedEnd();
        LocalDateTime realEnd = project.getRealEnd();

        return violations.isEmpty() &&
                plannedEnd.isAfter(start) &&
                (realEnd == null || realEnd.isAfter(start));
    }

    public boolean areAllocationTimeRangesOverlapping(
            LocalDateTime start,
            LocalDateTime plannedEnd,
            List<Allocation> allocations
    ){
        long startUnix = start.toEpochSecond(ZoneOffset.UTC);
        long plannedEndUnix = plannedEnd.toEpochSecond(ZoneOffset.UTC);
        return allocations.stream()
                .map(Allocation::getProject)
                .anyMatch(allocatedProject -> {
                    long allocatedStart = allocatedProject.getStart().toEpochSecond(ZoneOffset.UTC);
                    long allocatedPlannedEnd = allocatedProject.getPlannedEnd().toEpochSecond(ZoneOffset.UTC);
                    return Math.max(startUnix, allocatedStart) <= Math.min(plannedEndUnix, allocatedPlannedEnd);
                });
    }

}
