package de.hmmh.pmt.project;

import de.hmmh.pmt.IntegrationTest;
import de.hmmh.pmt.db.Project;
import org.junit.jupiter.api.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.web.client.RestClientException;


import java.util.Map;


import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class GetAllEmployeesByProjectTest extends IntegrationTest {

    @Test
    void shouldReturnNotFoundWhenProjectDoesNotExist() throws Exception {
        createTestProjectData();

        this.mvc
                .perform(getRequest(50L))
                .andExpect(status().isNotFound());
    }

    @Test
    void shouldReturnOkWhenProjectHasNoEmployees() throws Exception {
        Map<String, Project> allProjects = createTestProjectData();

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId()))
                .andExpect(status().isOk());
    }

    @Test
    void shouldReturnListOfEmployeesWhenAllParametersAreValid() throws Exception {
        Map<String, Project> allProjects = createTestProjectData();
        createTestAllocationData(allProjects);

        this.mvc
                .perform(getRequest(allProjects.get("ai-research").getId()))
                .andExpect(status().isOk());
    }

    @Test
    void shouldReturnInternalServer() throws Exception {
        when(this.mockEmployeeApi.findAll1())
                .thenThrow(new RestClientException("Internal Server Error"));

        Map<String, Project> allProjects = createTestProjectData();
        createTestAllocationData(allProjects);

        this.mvc
                .perform(getRequest(allProjects.get("ai-research").getId()))
                .andExpect(status().isInternalServerError());
    }

    private RequestBuilder getRequest(Long projectId) {
        return MockMvcRequestBuilders
                .get(baseUri + "/project/" + projectId + "/employees")
                .contentType(MediaType.APPLICATION_JSON);
    }
}
